:- module(abs_interval_rt,
	    [
		'abs'/2,
		constraint/2,
		'.<-.'/2,
		'.->.'/2,
		'abs_atom'/1,
		'abs_is'/2,
		'.=.'/2,
		'.<.'/2,
		'.>.'/2,
		'.=:=.'/2,
		'.@<.'/2,
		'.@>.'/2,
		'.==.'/2,
		'join'/3,
		'meet'/3,
		'in'/2,
		put_abs_list/2,
		get_abs_list/2,
		put_abs_list_const/2,
		get_abs_list_const/2,
		check_if_bot/1,
		make_bot/1,
		join_info_att/2,


		% dump_attributes/3,
		% current_space/1,
		% project_gen_space/4,
		% project_answer_space/4,
		% entail/5,
		% reinstall_space/4,
		% apply_answer/4,

		aux_dump_attributes/3,
		aux_entail/4,
		aux_apply_answer/3,
		aux_clean_vars/2,

		entailment/2,
		entailment_list/2,
		entailment_constraints/4,
		join_abs_list/2,
		extract_const_domain/4,
		make_proyection/3,
		simplify_iguality/4,
		in/4
	    ]).


:- use_package(attr).
:- use_module(engine(attributes)).

:- use_module(library(aggregates)).
:- use_module(library(format)).
:- use_module(library(terms_vars)).
:- use_module(library(sets)).

%:- use_module(abstraction).
:- use_module(interval_abstraction).

:- include(abs_interval_syntax).

add_constraint(V1, _, _AbsX) :-
	ground(V1), !.
add_constraint(V1, Constraints, NewAbs) :-
	(
	    get_attr_local(V1, '$abs'(PrevConst, Abs)) ->
	    insert_constraint(V1, Constraints, PrevConst, FinalCosnt),
	    abs_meet(NewAbs, Abs, FinalAbs),
	    FinalAbs \= bot,
	    put_attr_local(V1, '$abs'(FinalCosnt, FinalAbs)),
	    (
		Abs == FinalAbs ->
		true
	    ;
		%		print('|'),
		check_constraint(PrevConst)
	    )
	;
	    in_abs_domain(NewAbs, _),
	    insert_constraint(V1, Constraints, [], FinalConst),
	    put_attr_local(V1, '$abs'(FinalConst, NewAbs))
	).

insert_list_constraint(_,  [],                Rs,        Rs).
insert_list_constraint(V1, [Constraint|Rest], PrevConst, FinalConst) :-
	insert_constraint(V1, Constraint, PrevConst, TempConst),
	insert_list_constraint(V1, Rest, TempConst, FinalConst).
insert_constraint(V1, Constraint, Rs, Rs) :-
	varset(Constraint-V1, Xs),
	Xs == [V1], !.
insert_constraint(_, Constraint, Rs, Final) :-
	insert(Rs, Constraint, Final).
check_constraint([]) :- !.
check_constraint([C|Cs]) :-
	call(C),
	check_constraint(Cs).


%% OPERATORS $$

constraint(Constant, []) :-
	ground(Constant), !.
constraint(V1, Const) :-
	(
	    get_attr_local(V1, '$abs'(Const, _)) ->
	    true
	;
	    Const = []
	).

abs(Constant, AbsX) :-
	ground(Constant), !,
	(
	    abstract_constant(Constant, AbsX) ->
	    true
	;
	    fail
	).
abs(X, NewAbsX) :-
	ground(NewAbsX), !,
	(
	    get_attr_local(X, '$abs'(PrevConst, Abs)) ->
	    abs_meet(NewAbsX, Abs, FinalAbs),
	    FinalAbs \= bot,
	    put_attr_local(X, '$abs'(PrevConst, FinalAbs)),
	    (
		Abs == FinalAbs ->
		true
	    ;
		%		true
		check_constraint(PrevConst)
	    )
	;
	    (
		NewAbsX == top ->
		put_attr_local(X, '$abs'([], ['-oo','oo']))
	    ;
		in_abs_domain(NewAbsX, _),
		put_attr_local(X, '$abs'([], NewAbsX))
	    )
	).
abs(X, AbsX) :-
	(
	    get_attr_local(X, '$abs'(_, PrevAbs)) ->
	    AbsX = PrevAbs
	;
	    AbsX = ['-oo','oo'],
	    put_attr_local(X, '$abs'([], AbsX))
	).


X .<-. Abs :- abs(X, Abs).
X .->. Abs :- abs(X, Abs).


abs_atom(V1) :-
	abs(V1, atm).

V1 'abs_is' E :-
	nonvar(E),
	E =.. [Op, S1, S2],
	abs(V1, AbsV1),
	abs(S1, AbsS1),
	abs(S2, AbsS2),
	abs_operation(Op, AbsV1, AbsS1, AbsS2, [NewAbsV1, NewAbsS1, NewAbsS2]),
	add_constraint(V1, V1 'abs_is' E, NewAbsV1),
	add_constraint(S1, V1 'abs_is' E, NewAbsS1),
	add_constraint(S2, V1 'abs_is' E, NewAbsS2).

V1 .=. V2 :-
	abs(V1, AbsV1),
	abs(V2, AbsV2),
	abs_asignation(=, AbsV1, AbsV2, [NewAbsV1, NewAbsV2]),
	add_constraint(V1, V1 .=. V2, NewAbsV1),
	add_constraint(V2, V1 .=. V2, NewAbsV2).

V1 .<. V2 :-
	abs(V1, AbsV1),
	abs(V2, AbsV2),
	abs_condition(<, AbsV1, AbsV2, [NewAbsV1, NewAbsV2]),
	add_constraint(V1, V1 .<. V2, NewAbsV1),
	add_constraint(V2, V1 .<. V2, NewAbsV2).
V1 .>. V2 :-
	abs(V1, AbsV1),
	abs(V2, AbsV2),
	abs_condition(>, AbsV1, AbsV2, [NewAbsV1, NewAbsV2]),
	add_constraint(V1, V1 .>. V2, NewAbsV1),
	add_constraint(V2, V1 .>. V2, NewAbsV2).

V1 .=:=. V2 :-
	abs(V1, AbsV1),
	abs(V2, AbsV2),
	abs_condition(=:=, AbsV1, AbsV2, [NewAbsV1, NewAbsV2]),
	add_constraint(V1, V1 .=:=. V2, NewAbsV1),
	add_constraint(V2, V1 .=:=. V2, NewAbsV2).

%% var @< num @< atm @< str %%
V1 .@<. V2 :-
	abs(V1, AbsV1),
	abs(V2, AbsV2),
	abs_condition(@<, AbsV1, AbsV2, [NewAbsV1, NewAbsV2]),
	add_constraint(V1, V1 .@<. V2, NewAbsV1),
	add_constraint(V2, V1 .@<. V2, NewAbsV2).
V1 .@>. V2 :-
	V2 .@<. V1.

V1 .==. V2 :-
	abs(V1, AbsV1),
	abs(V2, AbsV2),
	abs_condition(==, AbsV1, AbsV2, [NewAbsV1, NewAbsV2]),
	add_constraint(V1, V1 .==. V2, NewAbsV1),
	add_constraint(V2, V1 .==. V2, NewAbsV2).


%% lattice constraint - 
V1 in V2 :-
	abs(V1, AbsV1),
	abs(V2, AbsV2),
	abs_meet(AbsV1, AbsV2, NewAbsV1),
	add_constraint(V1, V1 in V2, NewAbsV1),
	add_constraint(V2, V1 in V2, AbsV2).

%% join /\ lub
join(A, B, S) :-
	abs(A, AbsA),
	abs(B, AbsB),
	abs_join(AbsA, AbsB, AbsS),
	AbsS \= bot,
	abs(S, AbsS).

%% meet \/ glb
meet(A, B, S) :-
	abs(A, AbsA),
	abs(B, AbsB),
	abs_meet(AbsA, AbsB, AbsS),
	AbsS \= bot,
	abs(S, AbsS).

%% Extras %%
join_abs_list([X],       X).
join_abs_list([X, Y|Rs], List) :-
	join_abs_list_aux(X, Y, Join),
	join_abs_list([Join|Rs], List).
join_abs_list_aux([],      [],      []).
join_abs_list_aux([X1|Xs], [Y1|Ys], [J|Js]) :-
	abs_join(X1, Y1, J),
	join_abs_list_aux(Xs, Ys, Js).
%% Extras %%



%% attr pedricates %%
:- multifile attr_portray_hook/2.
:- multifile attr_unify_hook/2. %% needed
:- multifile attribute_goals/3.

attr_portray_hook(Att, Var) :-
	display(Var), display(', '), display(Att).

attr_unify_hook('$abs'(ConstX, AbsX), Y) :-
	get_attr_local(Y, '$abs'(ConstY, AbsY)),
	abs_meet(AbsX, AbsY, FinalAbs),
	ord_union(ConstX, ConstY, FinalConst),
	put_attr_local(Y, '$abs'(FinalConst, FinalAbs)),
	% check_constraint(FinalConst).
	(
	    AbsX == AbsY ->
	    true
	;
	    AbsX = FinalAbs ->
	    check_constraint(ConstY)
	;
	    AbsY = FinalAbs ->
	    check_constraint(ConstX)
	;
	    check_constraint(FinalConst)
	).

attribute_goals(X, [S|T], T) :-
	get_attr_local(X, '$abs'(Consts, Dom)),
	S = abs([X, Consts, Dom]).


%% Entailment X |- Y
entailment(X, Y) :-
	meet(X, Y, X).


entailment_list([],        []).
entailment_list([X|RestX], [Y|RestY]) :-
	entailment(X, Y),
	entailment_list(RestX, RestY).

entailment_constraints(const(CsX, ConstX), domain(Vs, DomX), const(CsY, ConstY),
	    domain(Vs, DomY)) :-
	entailment_abs_list(DomX, DomY),
	sort(CsY,    Sy),
	sort(ConstY, Sconsty),
	display(CsX), display(ConstX), nl,
	display(CsY), display(ConstY), nl,
	display(Sy), display(Sconsty), nl,
	CsX = Sy, nl,
	display(CsX), display(ConstX), nl,
	display(CsY), display(ConstY), nl,
	display(Sy), display(Sconsty), nl,
	entailment_const(ConstX, ConstY).

entailment_abs_list([],     []).
entailment_abs_list([X|Xs], [Y|Ys]) :-
	abs_meet(X, Y, X),
	entailment_abs_list(Xs, Ys).

entailment_const([],     []).
entailment_const(ConstX, ConstY) :-
	ord_subtract(ConstY, ConstX, []).

% entailment_constraints(const([X,T], [X .<. T]), domain([X], [pos]), const([Y,R], [Y .<. R]), domain([Y], [pos+z])).

% entailment_constraints(const([X,Y], [X .<. Y]), domain([X, Y], [neg, zero]), const([A, B], [A .<. B]), domain([X, Y], [num, num])).

% dump_constraints(Vs,  const(Cs, Constraints), domain(Vs,Dom)) :-
% 	abs_list(Vs, Dom),
% 	constraint_list(Vs, Constraints),
% 	term_variables(Constraints, Cs).

% dump_attributes(Vs, Constraints, domain(NewVs, Dom)) :-
% 	copy_term(Vs, NewVs, Copy),
% 	extract_const_domain(Copy, NewVs, Constraints, Dom),
% 	make_proyection(NewVs, Constraints, ProyectStore),
% 	display(ProyectStore).


extract_const_domain(_,                      [],      [],    []).
extract_const_domain([abs([V1, C1, D1])|Rs], [V2|Vs], Const, [D1|Ds]) :-
	V1 == V2,
	extract_const_domain(Rs, Vs, Cs, Ds),
	ord_union(C1, Cs, Const).
extract_const_domain([abs([V1, C1, _])|Rs], [V2|Vs], Const, Domain) :-
	V1 \== V2,
	extract_const_domain(Rs, [V2|Vs], Cs, Domain),
	ord_union(C1, Cs, Const).


make_proyection(Vs, Constraints, Proyection) :-
	varset(Constraints, Ts),
	sort(Vs, SVs),
	ord_subtract(Ts, SVs, NoVs),
	simplify_iguality(NoVs, Constraints, [], Proyection).

simplify_iguality(_,    [],                    R,    R).
simplify_iguality(NoVs, [C .=. V|Constraints], Prev, Proyection) :- !,
	(
	    in(C, NoVs, [], NewNoVs) ->
	    C = V,
	    simplify_iguality(NewNoVs, Constraints, Prev, Proyection)
	;
	    in(V, NoVs, [], NewNoVs2) ->
	    C = V,
	    simplify_iguality(NewNoVs2, Constraints, Prev, Proyection)
	;
	    insert(Prev, C .=. V, NewPrev),
	    simplify_iguality(NoVs, Constraints, NewPrev, Proyection)
	).
simplify_iguality(NoVs, [C|Constraints], Prev, Proyection) :-
	insert(Prev, C, NewPrev),
	simplify_iguality(NoVs, Constraints, NewPrev, Proyection).

in(V1, [V2|Cs], Prev, Final) :-
	V1 == V2,
	append(Prev, Cs, Final).
in(V1, [V2|Cs], Prev, Final) :-
	V1 \== V2,
	in(V1, Cs, [V2|Prev], Final).






% abs_list([],[]).
% abs_list([V|Vs], [D|Ds]) :-
% 	abs(V,D),
% 	abs_list(Vs,Ds).

% constraint_list([],[]).
% constraint_list([V|Vs], Const) :-
% 	constraint_list(Vs,Consts),
% 	constraint(V, Cs),
% 	ord_union(Consts, Cs, Const).

% dump_attributes(Vs, Const, domain(Vs, Dom)) :-
% 	abs_list(Vs, Dom),
% %	detach_list(Vs),
% 	Const = [9 .=. 9].
% detach_list([]).
% detach_list([X | Xs]) :-
% 	detach_attribute(X),
% 	detach_list(Xs).


%% To check error in copy_cross
% dump_attributes(Vs, [], Dom) :-
% 	copy_term(Vs, NewVs, Copy),
% 	extract_const_domain(Copy, NewVs, _Constraints, Dom).
% dump_attributes(Vs, Constraints, p(Vs,Dom)) :-
% 	copy_term(Vs, NewVs, Copy),
% 	extract_const_domain(Copy, NewVs, Constraints, Dom).
% apply_answer([],_,p(_,[])).
% apply_answer([X|Xs], _, p(_,[D|Ds])) :-
%  	check_answer(X, D),
%  	apply_answer(Xs, _, p(_,Ds)).

%% Correct one



% entail(Dx, Dy) :-
% 	entail_list(Dx, Dy).



% entail(domain(_Vs, DomX), domain(_Vs, DomY)) :-
% %	print(['entail',DomX, DomY]),nl,
% 	entailment_abs_list(DomX, DomY).
% %	print('ok'),nl.


% ?- T .<. 0, X .=.Y , X .<. Y, dump_constraints([T,X],P,M).
% |||
% M = abs_list([T,X],[neg,num]),
% P = [X.<.Y,X.=.Y],
% abs(T,'$abs'([],neg)),
% abs(X,'$abs'([X.<.Y,X.=.Y],num)),
% abs(Y,'$abs'([X.<.Y,X.=.Y],num)) ? ;

% ?- abs(A,num), undump_constraint([A,B], [X.<.Y,X.=.Y,T.<.K], abs_list([T,X],[neg,num])).
% ||
% T = A,
% X = B,
% abs(A,'$abs'([A.<.K],neg)),
% abs(B,'$abs'([B.<.Y,B.=.Y],num)),
% abs(Y,'$abs'([B.<.Y,B.=.Y],num)),
% abs(K,'$abs'([A.<.K],num)) ? ;

% Z abs_is X + Y, X .>. W, dump_constraints([X,Y], C, D), copy_term([C,D],[E,F]), undump_constraint([A,B], E,F), abs(Y,zero), abs(Z,zero), abs(A,zero).




%%

join_info_att([A], Copy) :-
	copy_list(A, Copy).
join_info_att([A, B|Rest], Rs) :-
	join_info_list(A, B, Join),
	join_info_att([Join|Rest], Rs).
join_info_list([],     [],     []).
join_info_list([A|As], [B|Bs], [Join|Js]) :-
	join(A, B, Join),
	join_info_list(As, Bs, Js).
copy_list([],       []).
copy_list([A|Rest], [Copy|RestCopy]) :-
	abs(A,    AbsA),
	abs(Copy, AbsA),
	copy_list(Rest, RestCopy).



put_abs_list([],     []).
put_abs_list([V|Vs], [abs(V, AbsV)|AbsList]) :-
	var(V),
	(
	    ground(AbsV) ->
	    abs(V, AbsV)
	;
	    abs(V, var)
	    %	    put_attr_local(V, '$abs'([], [var]))
	),
	put_abs_list(Vs, AbsList).

get_abs_list([],     []).
get_abs_list([V|Vs], [abs(V, AbsV)|AbsList]) :-
	(
	    get_attr_local(V, '$abs'(_, AbsV)) ->
	    detach_attribute(V)
	;
	    AbsV = var
	),
	get_abs_list(Vs, AbsList).

put_abs_list_const([],     []).
put_abs_list_const([V|Vs], [abs(V, Const, AbsV)|AbsList]) :-
	(
	    var(V) ->
	    put_attr_local(V, '$abs'(Const, AbsV))
	;
	    true
	),
	put_abs_list_const(Vs, AbsList).

get_abs_list_const([],     []).
get_abs_list_const([V|Vs], [abs(V, Const, AbsV)|AbsList]) :-
	(
	    get_attr_local(V, '$abs'(Const, [AbsV])) ->
	    detach_attribute(V)
	;
	    Const = [],
	    AbsV = var
	),
	get_abs_list_const(Vs, AbsList).


% check_if_bot(Body) :-
% 	varset(Body, Xs),
% 	member(X, Xs),
% 	get_attr_local(X, '$abs'(_, [bot])).
check_if_bot(Vars) :-
	member(V1, Vars),
	abs(V1, AbsV1),
	AbsV1 == bot, !.

make_bot([]).
make_bot([V|Rest]) :-
	put_attr_local(V, '$abs'([], bot)),
	make_bot(Rest).








%% Aux predicate for tabling interface:

%% DUMP ATTRIBUTES %%
aux_dump_attributes(Vars, Space, Dom) :-
	dump_space(Vars, Space),
	dump_domain(Vars, Dom).
dump_space(Vars, Space) :-
	project_space(Vars, Space).
dump_domain([], []).
dump_domain([X|Xs], [D|Ds]) :-
	abs(X, D),
%	put_attr_local(X, '$abs'([], Dom)),
	dump_domain(Xs, Ds).


%% ENTAIL %%
aux_entail(SpaceA, DomainA, SpaceB, DomainB) :-
	entail_domain(DomainA, DomainB),
	entail_space(SpaceA, SpaceB).
entail_domain([], []).
entail_domain([X|Xs], [Y|Ys]) :-
	abs_meet(X, Y, X),
	entail_domain(Xs, Ys).
entail_space([Vars,SpaceA], [Vars,SpaceB]) :-
	sort(SpaceA, SortA),
	sort(SpaceB, SortB),
	ord_subtract(SortA, SortB, []).

%% APPLY ANSWER %%
aux_apply_answer(Vars, Space, Dom) :-
	apply_domain(Vars, Dom),
	apply_space(Vars, Space).
apply_domain([], []).
apply_domain([X|Xs], [D|Ds]) :-
	check_answer(X, D),
	apply_domain(Xs, Ds).
apply_space(_, [_,[]]) :- !.
apply_space(Vars, [Vars, Cosntraints]) :-
	call_constraints(Cosntraints).
call_constraints([]).
call_constraints([X |Xs]) :-
	call(X),
	call_constraints(Xs).
% :- use_module(library(attr/attr_rt)).
check_answer(X, NewAbsX) :-
	var(X),
	get_attribute(X, att(X, _, [])), !,
	put_attr_local(X, '$abs'([], NewAbsX)).
check_answer(X, NewAbsX) :-
	get_attr_local(X, '$abs'(Const, AbsX)), !,
	abs_meet(NewAbsX, AbsX, NewAbsX),
	put_attr_local(X, '$abs'(Const, NewAbsX)).
check_answer(X, NewAbsX) :-
	put_attr_local(X, '$abs'([], NewAbsX)).


%% PROJECTION SPACE LATTICE (V1 in V2) %%
aux_project_space(Vars, _, _, Space) :-
	project_space(Vars, Space).

aux_clean_vars(Vars, [Vars,Space]) :-
	clean_constraints(Vars),
	call_constraints(Space).
clean_constraints([]).
clean_constraints([X|Xs]) :-
	get_attr_local(X, '$abs'(_, AbsX)),
	(
	    ground(AbsX) ->
	    put_attr_local(X, '$abs'([], AbsX))
	;
	    put_attr_local(X, '$abs'([], ['-oo','oo']))
	),
	clean_constraints(Xs).
	


project_space(Vars, [Vars,Space]) :-
	sort(Vars, V),
	project(V, Others, Space1),
	simplifier(Space1, Others, Space).

simplifier(Space1, [],     Space1).
simplifier(Space1, [O|Os], FinalSpace) :-
	split_space(Space1, O, SpaceLeft, SpaceRight, NewSpace),
	new_constraint(SpaceLeft, SpaceRight, NewConstraints),
	ord_union(NewSpace, NewConstraints, NextSpace),
	simplifier(NextSpace, Os, FinalSpace).

new_constraint([],            _,  []).
new_constraint([in(_, V)|Ls], Rs, NewC) :-
	new_constraint_aux(V, Rs, NewC0),
	new_constraint(Ls, Rs, NewC1),
	ord_union(NewC0, NewC1, NewC).

new_constraint_aux(_, [],             []).
new_constraint_aux(V, [in(V2, _)|Rs], [in(V2, V)|Ss]) :-
	V \== V2, !,
	new_constraint_aux(V, Rs, Ss).
new_constraint_aux(V, [_|Rs], Ss) :-
	new_constraint_aux(V, Rs, Ss).


split_space([],             _O, [],             [], []).
split_space([in(O1, V)|Ss], O,  [in(O1, V)|Ls], Rs, Ns) :-
	O1 == O, !,
	split_space(Ss, O, Ls, Rs, Ns).
split_space([in(V, O1)|Ss], O, Ls, [in(V, O1)|Rs], Ns) :-
	O1 == O, !,
	split_space(Ss, O, Ls, Rs, Ns).
split_space([C|Ss], O, Ls, Rs, [C|Ns]) :-
	split_space(Ss, O, Ls, Rs, Ns).

project(Vars, TotalOthers, Space) :-
	take_in_const(Vars, C0),
	others(Vars, C0, Others),
	project_aux(Vars, C0, Others, [], TotalOthers, Space).

project_aux(_,    C,  [],      O,         O,           C) :- !.
project_aux(Vars, C0, Others0, PreOthers, TotalOthers, Space) :-
	take_in_const(Others0, C1),
	ord_union(C0,   C1,        C2),
	ord_union(Vars, PreOthers, Total0),
	others(Total0, C2, Others1),
	ord_union(PreOthers, Others1, PreOthers1),
	project_aux(Vars, C2, Others1, PreOthers1, TotalOthers, Space).

take_in_const([],     []).
take_in_const([X|Xs], Const) :-
	constraint(X, Cs),
	filter_in(Cs, In_Cs),
	take_in_const(Xs, CXs),
	ord_union(CXs, In_Cs, Const).

filter_in([],     []).
filter_in([X|Xs], [X|Fs]) :-
	\+ \+ X =.. [in, _A, _B], !,
	filter_in(Xs, Fs).
filter_in([_X|Xs], Fs) :-
	filter_in(Xs, Fs).

others(Vars, InConst, Other) :-
	varset(InConst, Total),
	ord_subtract(Total, Vars, Other).
%% PROJECTION SPACE LATTICE (V1 in V2) %%



	


%% CHECK EXAMPLES %%

% ?- aux_entail(([X,Y],[X in Y]), [pos,pos], ([A,B],[A in B]), [pos,pos], _).
% X = A,
% Y = B ? ;
% ?- aux_entail(([X,Y],[X in Y]), [neg,pos], ([A,B],[A in B]), [pos,pos], _).
% no
% ?- aux_entail(([X,Y],[X in Y]), [neg,pos], ([A,B],[A in B]), [num,pos], _).
% X = A,
% Y = B ? ;
% ?- aux_entail(([X,Y],[X in Y]), [neg,pos], ([A,B],[B in A]), [num,pos], _).
% no
% ?- aux_entail(([X,Y],[X in Y]), [neg,pos], ([A,B],[]), [num,pos], _).
% no
% ?- aux_entail(([X,Y],[]), [neg,pos], ([A,B],[B in A]), [num,pos], _).
% X = A,
% Y = B ? ;

%% abs(X,pos), X in Y, aux_dump_attributes([X,Y], S1, D1), A in B, aux_dump_attributes([A,B], S, D), aux_entail(S1, D1, S, D, _).

%%  abs(X,zero), aux_apply_answer([X,Y], ([A,B], [A in B]), [zero,pos+z], _), B .=.0.
