:- module(interval_abstraction, [
		in_abs_domain/2,
		abstract_constant/2,
		abs_operation/5,
		abs_asignation/4,
		abs_condition/4,
		abs_join/3,
		abs_meet/3,
		calculate/4,
		abs_join_list/2
	    ]).


% :- use_package(tabling).

% %:- table in_abs_domain/2, abstract_constant/2, abs_operation/5.
% :- table in_abs_domain/2, abs_operation/5.
% :- table abs_asignation/4, abs_condition/4, abs_join/3, abs_meet/3, calculate/4.

:- discontiguous true_table/1, abs_operation/5, calculate/4.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Abstractions
in_abs_domain(X, _) :- member(X, [bot]).
in_abs_domain([A,B], interval) :- check_abs([A,B], _).

%% Abstraction of constants
abstract_constant(N, [N,N]) :- num(N), N >= -100, N =< 100.
abstract_constant(N, ['-oo', -101]) :- num(N), N =< -101.
abstract_constant(N, [101, 'oo']) :- num(N), N >= 101.
%abstract_constant(I, Check) :- check_abs(I, Check), !.
abstract_constant(N, bot) :- \+ num(N).

check_abs([A,B], [C,D]) :-
	(
	    A = '-oo' ->
	    C = '-oo'
	;
	    num(A), A =< -101 ->
	    C = '-oo'
	;
	    num(A), A >= 101 ->
	    C = 101
	;
	    C = A
	),
	(
	    B = 'oo' ->
	    D = 'oo' 
	;
	    num(B), B >= 101 ->
	    D = 'oo'
	;
	    num(B), B =< -101 ->
	    D = -101
	;
	    D = B
	),
	less_eq(C, D).



%% Abstract operations. 
abs_operation(+, [A1, A2], [B1, B2], [C1, C2], [[NewA1, NewA2], [B1, B2], [C1, C2]]) :-
	calculate(+, TA1, B1, C1),
	calculate(+, TA2, B2, C2),
	check_abs([TA1, TA2], [NewA1, NewA2]),
	abs_meet([A1, A2], [NewA1, NewA2], [NewA1, NewA2]).

calculate(+, 'oo', 'oo', O2) :- num(O2).
calculate(+, 'oo', O1, 'oo') :- num(O1).
calculate(+, 'oo', 'oo', 'oo').
calculate(+, '-oo', '-oo', O2) :- num(O2).
calculate(+, '-oo', O1, '-oo') :- num(O1).
calculate(+, '-oo', '-oo', '-oo').
calculate(+, S, O1, O2) :- num(O1), num(O2), TempS is O1 + O2,
	(
	    TempS =< -101 ->
	    S = '-oo'
	;
	    TempS >= 101 ->
	    S = 'oo'
	;
	    S = TempS
	).
	

abs_operation(*, [A1, A2], [B1, B2], [C1, C2], [[NewA1, NewA2], [B1, B2], [C1, C2]]) :-
	calculate(*, R1, B1, C1),
	calculate(*, R2, B1, C2),
	calculate(*, R3, B2, C1),
	calculate(*, R4, B2, C2),
	min([R1,R2,R3,R4], TA1),
	max([R1,R2,R3,R4], TA2),
	check_abs([TA1, TA2], [NewA1, NewA2]),
	abs_meet([A1, A2], [NewA1, NewA2], [NewA1, NewA2]).

min([Min], Min).
min([A1,A2|As], Min) :-
	less(A1, A2), !,
	min([A1|As], Min).
min([_,A2|As], Min) :-
	min([A2|As], Min).
max(['oo'|_], 'oo') :- !.
max([_,'oo'|_], 'oo') :- !.
max([Max], Max).
max([A1,A2|As], Max) :-
	less(A1, A2), !,
	max([A2|As], Max).
max([A1,_|As], Max) :-
	max([A1|As], Max).

calculate(*, 'oo', 'oo', O2) :- num(O2), O2 > 0.
calculate(*, 0, 'oo', O2) :- num(O2), O2 = 0.
calculate(*, '-oo', 'oo', O2) :- num(O2), O2 < 0.
calculate(*, 'oo', O1, 'oo') :- num(O1), O1 > 0.
calculate(*, 0, O1, 'oo') :- num(O1), O1 = 0.
calculate(*, '-oo', O1, 'oo') :- num(O1), O1 < 0.
calculate(*, 'oo', 'oo', 'oo').
calculate(*, '-oo', '-oo', O2) :- num(O2), O2 > 0.
calculate(*, 0, '-oo', O2) :- num(O2), O2 = 0.
calculate(*, 'oo', '-oo', O2) :- num(O2), O2 < 0.
calculate(*, '-oo', O1, '-oo') :- num(O1), O1 > 0.
calculate(*, 0, O1, '-oo') :- num(O1), O1 = 0.
calculate(*, 'oo', O1, '-oo') :- num(O1), O1 < 0.
calculate(*, 'oo', '-oo', '-oo').
calculate(*, S, O1, O2) :- num(O1), num(O2), TempS is O1 * O2,
	(
	    TempS =< -101 ->
	    S = '-oo'
	;
	    TempS >= 101 ->
	    S = 'oo'
	;
	    S = TempS
	).

abs_asignation(=, var,   AbsV2, [AbsV2, AbsV2]).
abs_asignation(=, AbsV1, var,   [AbsV1, AbsV1]) :-
	AbsV1 \= var.
abs_asignation(=, AbsV1, AbsV2, [NewAbsV1, NewAbsV1]) :-
	AbsV1 \= var, AbsV2 \= var,
	calculate(==, AbsV1, AbsV2, NewAbsV1).


abs_condition(=:=, AbsV1, AbsV2, [NewAbsV1, NewAbsV1]) :-
	abs_meet(AbsV1, AbsV2, NewAbsV1).
abs_condition(<, [A1,A2], [B1,B2], [NewAbsV1, [B1, B2]]) :-
	(
	    less(A2, B1) ->
	    NewAbsV1 = [A1, A2]
	;
	    B2 == 'oo' ->
	    NewAbsV1 = [A1, A2]
	;
	    NewA2 is B2 - 1,
	    less_eq(A1, NewA2),
	    NewAbsV1 = [A1, NewA2]
	).
abs_condition(>, [A1,A2], [B1,B2], [NewAbsV1, [B1, B2]]) :-
	(
	    less(B2, A1) ->
	    NewAbsV1 = [A1, A2]
	;
	    B1 == '-oo' ->
	    NewAbsV1 = [A1, A2]
	;
	    NewA1 is B1 + 1,
	    less_eq(NewA1, A2),
	    NewAbsV1 = [NewA1, A2]
	).


% join or lub - least upper bound
abs_join([A1,A2], [B1, B2], [C1, C2]) :-
	min([A1, B1], TC1),
	max([A2, B2], TC2),
	check_abs([TC1, TC2], [C1, C2]).

% meet ot glb - greatest lower bound
% equal true_table as ==
abs_meet([A1,A2], [B1,B2], [C1, C2]) :-
	max([A1, B1], TC1),
	min([A2, B2], TC2),
	check_abs([TC1, TC2], [C1, C2]).

less('-oo',_).
less(O1,'oo') :- num(O1).
less(O1, O2) :- num(O1), num(O2), O1 < O2.
	
less_eq(A,B) :- less(A,B), !.
less_eq(A,B) :- num(A), num(B), A =:= B.

abs_join_list([X], X).
abs_join_list([A, B| Ls], JoinList) :-
	abs_join_list_aux(A, B, Join),
	abs_join_list([Join| Ls], JoinList).
abs_join_list_aux([],[],[]).
abs_join_list_aux([A|As], [B|Bs], [J|Js]) :-
	abs_join(A, B, J),
	abs_join_list_aux(As, Bs, Js).
