:- module(abs_solver_rt,
	[
	    'abs'/2,
	    in/2,
	    '.<-.'/2,
	    '.->.'/2,
	    'abs_atom'/1,
	    'abs_is'/2,
	    '.=.'/2,
	    '.<.'/2,
	    '.>.'/2,
	    '.=:=.'/2,
	    '.@<.'/2,
	    '.@>.'/2,
	    '.==.'/2,
	    'join'/3,
	    'meet'/3,
	    put_abs_list/2,
	    get_abs_list/2,
	    take_abs_list/2,
	    check_no_bot/1,
	    make_bot/1,
	    join_info_att/2
	]).

:- use_package(attr).
:- use_module(engine(attributes)).

:- use_module(library(aggregates)).
:- use_module(library(format)).
:- use_module(library(terms_vars)).

:- use_module(library(abs_solver/solver_abstraction)).
:- include(library(abs_solver/abs_solver_syntax)).

% Check or give the abstract value of a ground-value
% Give the current abstraction of X or introduce a variable as abstract value
% if exist current abs value check if is equal to the new one.
% if not introduce the new abstract value
abs(Constant, AbsX) :-
	ground(Constant), !,
	(
	    abstract_constant(Constant, AbsX) ->
	    true
	;
	    fail
	).
abs(X, NewAbsX) :-
	ground(NewAbsX), !,
	NewAbsX \= bot,
	(
	    get_attr_local(X, AbsX) ->
	    (
		AbsX == var ->
		put_attr_local(X, NewAbsX)
	    ;
		abs_meet(NewAbsX, AbsX, FinalAbs),
		put_attr_local(X, FinalAbs)
	    )
	;
	    in_abs_domain(NewAbsX, _),
	    put_attr_local(X, NewAbsX)
	).
abs(X,AbsX) :-
	(
	    get_attr_local(X, PrevAbs) ->
	    AbsX = PrevAbs
	;
	    AbsX = top,
	    put_attr_local(X, AbsX)
	).


%% OPERATORS $$
V1 .<-. AbsV1 :- abs(V1, AbsV1).
V1 .->. AbsV1 :- abs(V1, AbsV1).

abs_atom(V1) :-
	abs(V1, atm).


V1 'abs_is' E :-
	nonvar(E),
	E =.. [Op, S1, S2],
	abs(V1, AbsV1),
	abs(S1, AbsS1),
	abs(S2, AbsS2),
	abs_operation(Op, AbsV1, AbsS1, AbsS2, [NewAbsV1, NewAbsS1, NewAbsS2]),
	abs(V1, NewAbsV1),
	abs(S1, NewAbsS1),
	abs(S2, NewAbsS2).

V1 .=. V2 :-
	abs(V1, AbsV1),
	abs(V2, AbsV2),
	abs_asignation(=, AbsV1, AbsV2, [NewAbsV1, NewAbsV2]),
	abs(V1, NewAbsV1),
	abs(V2, NewAbsV2).

V1 .<. V2 :-
	abs(V1, AbsV1),
	abs(V2, AbsV2),
	abs_condition(<, AbsV1, AbsV2, [NewAbsV1, NewAbsV2]),
	abs(V1, NewAbsV1),
	abs(V2, NewAbsV2).
V1 .>. V2 :-
	V2 .<. V1.

V1 .=:=. V2 :-
	abs(V1, AbsV1),
	abs(V2, AbsV2),
	abs_condition(=:=, AbsV1, AbsV2, [NewAbsV1, NewAbsV2]),
	abs(V1, NewAbsV1),
	abs(V2, NewAbsV2).

%% var @< num @< atm @< str %%
V1 .@<. V2 :-
	abs(V1, AbsV1),
	abs(V2, AbsV2),
	abs_condition(@<, AbsV1, AbsV2, [NewAbsV1, NewAbsV2]),
	abs(V1, NewAbsV1),
	abs(V2, NewAbsV2).
V1 .@>. V2 :-
	V2 .@<. V1.

V1 .==. V2 :-
	abs(V1, AbsV1),
	abs(V2, AbsV2),
	abs_condition(==, AbsV1, AbsV2, [NewAbsV1, NewAbsV2]),
	abs(V1, NewAbsV1),
	abs(V2, NewAbsV2).


%% lattice constraint - 
V1 in V2 :-
	abs(V1, AbsV1),
	abs(V2, AbsV2),
	abs_meet(AbsV1, AbsV2, NewAbsV1),
	abs(V1, NewAbsV1),
	abs(V2, AbsV2).



%% join /\ lub
join(A,B,S) :-
	abs(A, AbsA),
	abs(B, AbsB),
	abs_join(AbsA, AbsB, AbsS),
%	AbsS \= bot,
	abs(S, AbsS).

%% meet \/ glb
meet(A,B,S) :-
	abs(A, AbsA),
	abs(B, AbsB),
	abs_meet(AbsA, AbsB, AbsS),
	AbsS \= bot,
	abs(S, AbsS).


%% Extras %%
join_abs_list([X],       X).
join_abs_list([X, Y|Rs], List) :-
	join_abs_list_aux(X, Y, Join),
	join_abs_list([Join|Rs], List).
join_abs_list_aux([],      [],      []).
join_abs_list_aux([X1|Xs], [Y1|Ys], [J|Js]) :-
	abs_join(X1, Y1, J),
	join_abs_list_aux(Xs, Ys, Js).
%% Extras %%


:- multifile attr_portray_hook/2.
:- multifile attr_unify_hook/2.
:- multifile attribute_goals/3.

attr_portray_hook(Att,Var) :-
	display(Var), display(', '), display(Att).

attr_unify_hook(_Att, _X) :-
	true.

attribute_goals(X, [S | T], T) :-
	get_attr_local(X, Att),
 	S = (abs(X,Att)).

% Which is better ?
% X abs_is top ? .
% X.=.top ? .
% abs(X,top) ? .
% X<-top ? .


join_info_att([A], Copy) :-
	copy_list(A, Copy).
join_info_att([A, B | Rest], Rs) :-
	join_info_list(A, B, Join),
	join_info_att([Join | Rest], Rs).
join_info_list([], [], []).
join_info_list([A | As], [B | Bs], [Join | Js]) :-
	join(A, B, Join),
	join_info_list(As, Bs, Js).
copy_list([],[]).
copy_list([A | Rest], [Copy | RestCopy]) :-
	abs(A, AbsA),
	abs(Copy,AbsA),
	copy_list(Rest, RestCopy).

put_abs_list([],[]).
put_abs_list( [V | Vs], [abs(V, AbsV) | AbsList]) :-
	var(V),
	(
	    ground(AbsV) ->
	    put_attr_local(V, AbsV)
	;
	    put_attr_local(V, top)
	),
	put_abs_list(Vs, AbsList).

get_abs_list([],[]).
get_abs_list([V | Vs], [abs(V, AbsV) | AbsList]) :-
	(
	    get_attr_local(V, AbsV) ->
	    detach_attribute(V)
	;
	    AbsV = top
	),
	get_abs_list(Vs, AbsList).


take_abs_list([],[]).
take_abs_list([V | Vs], [AbsV | AbsList]) :-
	(
	    get_attr_local(V, AbsV) ->
	    true
	;
	    AbsV = top
	),
	take_abs_list(Vs, AbsList).


check_no_bot([]).
check_no_bot([X|Xs]) :-
	get_attr_local(X, Abs),
	Abs \= bot,
	check_no_bot(Xs).

make_bot([]).
make_bot([V |Rest]) :-
	put_attr_local(V, bot),
	make_bot(Rest).


%% Try to solve this
% ?- X .<. Y, abs(X, pos).
% abs(X,pos),
% abs(Y,num) ? .
% ?- abs(X,pos), X .<. Y.
% abs(X,pos),
% abs(Y,pos) ? .


% V1 abs_is E :-
% 	interp_arith_att(E, AbsE),
% 	abs(V1, AbsV1),
% 	(
% 	    AbsV1 \= var ->
% 	    abs_meet(AbsV1, AbsE, Abs_join)
% 	;
% 	    Abs_join = AbsE
% 	),
% 	(
% 	    abs_meet(Abs_join, AbsE, AbsR), AbsR \= AbsE ->
% 	    update_expresion_att(AbsV1, E)
% 	;
% 	    true
% 	),
% 	abs(V1, Abs_join).
% interp_arith_att(E, AbsE) :-
% 	E =.. [Op, S1, S2],
% 	abs(S1, AbsS1),
% 	abs(S2, AbsS2),
% 	abs_operation(Op, AbsS1, AbsS2, AbsE).
% interp_arith_att(Constant, AbsE) :-
% 	Constant =.. [_],
% 	abs(Constant, AbsE),
% 	in_abs_domain(AbsE,_).

% update_expresion_att(_AbsV1, Constant) :-
% 	Constant =.. [_].
% update_expresion_att(AbsV1, E) :-
% 	E =.. [Operator, S1, S2],
% 	abs(S1, AbsS1),
% 	abs(S2, AbsS2),
% 	findall(X, abs_operation(Operator, X, AbsS2, AbsV1), Xs),
% 	(
% 	    apply_glb(AbsS1, Xs, Ys),  join_list(Ys, Res), Res \= bot ->
% 	    abs(S1, Res)
% 	;
% 	    true
% 	).

% apply_glb(_, [], []).
% apply_glb(AbsV1, [Abs1 | Xs], [Res | Ys]) :-
% 	abs_meet(AbsV1, Abs1, Res),
% 	apply_glb(AbsV1, Xs, Ys).
% join_list([X], X).
% join_list([Abs1, Abs2 | As], Rs):-
% 	abs_join(Abs1, Abs2, Res),
% 	join_list([Res | As], Rs).