:- module(sign_rt, [
		abs_is/2,
		abs_atom/1,
		'.=.'/2,
		'.<.'/2,
		'.>.'/2,
		'.=:=.'/2,
		'.@<.'/2,
		'.@>.'/2,
		'.==.'/2
	    ]).


:- include(library(lattice/domain_example/sign_syntax)).


:- multifile
	domain_meet/3,
	domain_join/3,
	domain_constant/2,
	domain_operation/3,
	in_domain/2,
	
	operation/2.


abs_atom(V1) :- operation(atom, [V1]).

V1 .=. V2 :- operation(.=., [V1, V2]).
V1 .<. V2 :- operation(.<., [V1, V2]).
V1 .>. V2 :- operation(.<., [V2, V1]).
V1 .=:=. V2 :- operation(.=:=., [V1, V2]).
%% var @< num @< atm @< str %%
V1 .@<. V2 :- operation(.@<., [V1, V2]).
V1 .@>. V2 :- operation(.@<., [V2, V1]).
V1 .==. V2 :- operation(.==., [V1, V2]).

V1 'abs_is' E :-
	nonvar(E),
	E =.. [Op, S1, S2],
	operation(Op, [V1, S1, S2]).



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Abstractions
in_domain(X, _) :- member(X, [top, bot]).
in_domain(X, numbers) :- member(X, [neg, pos, zero, neg+z, pos+z, not_z,
		num]).
in_domain(var, variables).
in_domain(atm, atoms).
in_domain(str, structures).

%% Abstraction of constants
domain_constant(0, zero).
domain_constant(N, pos) :- num(N), N > 0.
domain_constant(N, neg) :- num(N), N < 0.
domain_constant(A, atm) :- atom(A).
domain_constant(S, str) :- struct(S).


%% Abstract operations. Use true tables.
domain_operation(atom, [_], [atm]).

domain_operation(Op, [AbsV1, AbsS1, AbsS2], NewAbstractions) :-
	domain_meet(AbsS1, num, NumS1),
	domain_meet(AbsS2, num, NumS2),
	calculate(Op, NumS1, NumS2, NewAbsV1),
	(
	    AbsV1 == var ->
	    NewAbstractions = [NewAbsV1, NumS1, NumS2]
	;
	    domain_meet(AbsV1, NewAbsV1, NewAbsV1) ->
	    NewAbstractions = [NewAbsV1, NumS1, NumS2]
	;
	    domain_meet(AbsV1, NewAbsV1, AbsV1) ->
	    findall([TempS1, TempS2],
		(
		    calculate(Op, TempS1, TempS2, AbsV1),
		    domain_meet(TempS1, NumS1, TempS1),
		    domain_meet(TempS2, NumS2, TempS2)
		),
		All),
	    (
		domain_join_list(All, [NewAbsS1, NewAbsS2]) ->
		NewAbstractions = [AbsV1, NewAbsS1, NewAbsS2]
	    ;
		NewAbstractions = [AbsV1, NumS1, NumS2]
	    )
	;
	    NewAbstractions = [bot, bot, bot]
	).


domain_operation(.=., [var,   AbsV2], [AbsV2, AbsV2]).
domain_operation(.=., [AbsV1, var],   [AbsV1, AbsV1]) :-
	AbsV1 \= var.
domain_operation(.=., [AbsV1, AbsV2], [NewAbsV1, NewAbsV1]) :-
	AbsV1 \= var, AbsV2 \= var,
	calculate(==, AbsV1, AbsV2, NewAbsV1).

% Restrict the domain subst of a variable depending on what it is
% compared with.  Will this be a lub or a glb?  Note: this does not
% assume any longer that the third argument is a constant (hence it
% can be num or bottom).  This makes it possible to express V < C and V > C.

% We have a variable with an abstract substitution and we check it
% against a constant.  We restrict the domain of the variable -- but
% we can also bo to bottom.  We have a glb here.  If we had
% zero-or-pos, zero-or-neg, not-zero the table would have been richer.
domain_operation(.=:=., [AbsV1, AbsV2], [NewAbsV1, NewAbsV1]) :-
	domain_meet(AbsV1, num, NumV1),
	domain_meet(AbsV2, num, NumV2),
	calculate(==, NumV1, NumV2, NewAbsV1).
domain_operation(.<., [AbsV1, AbsV2], [NewAbsV1, NewAbsV2]) :-
	domain_meet(AbsV1, num, NumV1),
	domain_meet(AbsV2, num, NumV2),
	calculate(@<, NumV1, NumV2, NewAbsV1),
	calculate(@>, NumV2, NumV1, NewAbsV2).



domain_operation(.==., [AbsV1, AbsV2], [NewAbsV1, NewAbsV1]) :-
	calculate(.==., AbsV1, AbsV2, NewAbsV1).
domain_operation(.@<., [AbsV1, AbsV2], [NewAbsV1, NewAbsV2]) :-
	calculate(@<, AbsV1, AbsV2, NewAbsV1),
	calculate(@>, AbsV2, AbsV1, NewAbsV2).

% join or lub - least upper bound
domain_join(AbsV1, AbsV2, Res) :-
	calculate(lub, AbsV1, AbsV2, Res).

% meet ot glb - greatest lower bound
% equal true_table as ==
domain_meet(Abs1, Abs2, Res) :-
	calculate(glb, Abs1, Abs2, Res).

% % Predicate for all the operators (also lub and glb) 
calculate(Operator, AbsA, AbsB, AbsOut) :-
	true_table([Operator, L1|Table]),
	member([AbsB|L2], Table),
	member(AbsA,      L1),
	calculate_aux(AbsA, L1, L2, Lout),
	member(AbsOut, Lout).
calculate_aux(_,    [],        [],     []).
calculate_aux(AbsA, [AbsA|L1], [X|L2], [X|LX]) :-
	calculate_aux(AbsA, L1, L2, LX).
calculate_aux(AbsA, [B|L1], [_L|L2], X) :-
	AbsA \= B,
	calculate_aux(AbsA, L1, L2, X).

% calculate_list(Operator, AbsA, AbsB, AbsOut) :-
% 	findall(r(TA, TB, TOut),
% 	(
% 	    domain_meet(AbsA, TA, TA),
% 	    domain_meet(AbsB, TB, TB),
% 	    calculate_(Operator, TA, TB, TOut)
% 	), All),
% 	All \= [],
% 	lub_list(All, List),
% 	List = r(AbsA, AbsB, AbsOut).

% lub_list([], r(bot, bot, bot)).
% lub_list([X],X).
% lub_list([r(X1,X2,X3), r(Y1,Y2,Y3)|Rs],List) :-
% 	calculate_(lub,X1,Y1,Lub1),
% 	calculate_(lub,X2,Y2,Lub2),
% 	calculate_(lub,X3,Y3,Lub3),
% 	lub_list([r(Lub1, Lub2, Lub3)|Rs], List).

domain_join_list([X],       X).
domain_join_list([A, B|Ls], JoinList) :-
	domain_join_list_aux(A, B, Join),
	domain_join_list([Join|Ls], JoinList).
domain_join_list_aux([],     [],     []).
domain_join_list_aux([A|As], [B|Bs], [J|Js]) :-
	domain_join(A, B, J),
	domain_join_list_aux(As, Bs, Js).

%% True Tables %%
true_table([+, [bot, neg, zero, pos, neg+z, not_z, pos+z, num, var, atm, str,
		    top],
		[bot,   bot, bot, bot,   bot, bot,   bot,   bot,   bot, bot,
		    bot, bot, bot],
		[neg,   bot, neg, neg,   num, neg,   num,   num,   num, bot,
		    bot, bot, num],
		[zero,  bot, neg, zero,  pos, neg+z, not_z, pos+z, num, bot,
		    bot, bot, num],
		[pos,   bot, num, pos,   pos, num,   num,   pos,   num, bot,
		    bot, bot, num],
		[neg+z, bot, neg, neg+z, num, neg+z, num,   num,   num, bot,
		    bot, bot, num],
		[not_z, bot, num, not_z, num, num,   not_z, num,   num, bot,
		    bot, bot, num],
		[pos+z, bot, num, pos+z, pos, num,   num,   pos+z, num, bot,
		    bot, bot, num],
		[num,   bot, num, num,   num, num,   num,   num,   num, bot,
		    bot, bot, num],
		[var,   bot, bot, bot,   bot, bot,   bot,   bot,   bot, bot,
		    bot, bot, bot],
		[atm,   bot, bot, bot,   bot, bot,   bot,   bot,   bot, bot,
		    bot, bot, bot],
		[str,   bot, bot, bot,   bot, bot,   bot,   bot,   bot, bot,
		    bot, bot, bot],
		[top,   bot, num, num,   num, num,   num,   num,   num, bot,
		    bot, bot, num]]).

true_table([*, [bot, neg, zero, pos, neg+z, not_z, pos+z, num, var, atm, str,
		    top],
		[bot,   bot, bot,   bot,  bot,   bot,   bot,   bot,   bot,  bot,
		    bot, bot, bot],
		[neg,   bot, pos,   zero, neg,   pos+z, not_z, neg+z, num,  bot,
		    bot, bot, num],
		[zero,  bot, zero,  zero, zero,  zero,  zero,  zero,  zero, bot,
		    bot, bot, num],
		[pos,   bot, neg,   zero, pos,   neg+z, not_z, pos+z, num,  bot,
		    bot, bot, num],
		[neg+z, bot, pos+z, zero, neg+z, pos+z, num,   neg+z, num,  bot,
		    bot, bot, num],
		[not_z, bot, not_z, zero, not_z, num,   not_z, num,   num,  bot,
		    bot, bot, num],
		[pos+z, bot, neg+z, zero, pos+z, neg+z, num,   pos+z, num,  bot,
		    bot, bot, num],
		[num,   bot, num,   zero, num,   num,   num,   num,   num,  bot,
		    bot, bot, num],
		[var,   bot, bot,   bot,  bot,   bot,   bot,   bot,   bot,  bot,
		    bot, bot, bot],
		[atm,   bot, bot,   bot,  bot,   bot,   bot,   bot,   bot,  bot,
		    bot, bot, bot],
		[str,   bot, bot,   bot,  bot,   bot,   bot,   bot,   bot,  bot,
		    bot, bot, bot],
		[top,   bot, num,   num,  num,   num,   num,   num,   num,  bot,
		    bot, bot, num]]).

% == ,[As...],
% B  , X1
% ...
% means A == B -> newSubsA = X1. Also newSubsB = X1
true_table([==, [bot, neg, zero, pos, neg+z, not_z, pos+z, num, var, atm, str,
		    top],
		[bot,   bot, bot, bot,  bot, bot,   bot,   bot,   bot,   bot,
		    bot, bot, bot],
		[neg,   bot, neg, bot,  bot, neg,   neg,   bot,   neg,   bot,
		    bot, bot, neg],
		[zero,  bot, bot, zero, bot, zero,  bot,   zero,  zero,  bot,
		    bot, bot, zero],
		[pos,   bot, bot, bot,  pos, bot,   pos,   pos,   pos,   bot,
		    bot, bot, pos],
		[neg+z, bot, neg, zero, bot, neg+z, neg,   zero,  neg+z, bot,
		    bot, bot, neg+z],
		[not_z, bot, neg, bot,  pos, neg,   not_z, pos,   not_z, bot,
		    bot, bot, not_z],
		[pos+z, bot, bot, zero, pos, zero,  pos,   pos+z, pos+z, bot,
		    bot, bot, pos+z],
		[num,   bot, neg, zero, pos, neg+z, not_z, pos+z, num,   bot,
		    bot, bot, num],
		[var,   bot, bot, bot,  bot, bot,   bot,   bot,   bot,   var,
		    bot, bot, var],
		[atm,   bot, bot, bot,  bot, bot,   bot,   bot,   bot,   bot,
		    atm, bot, atm],
		[str,   bot, bot, bot,  bot, bot,   bot,   bot,   bot,   bot,
		    bot, str, str],
		[top,   bot, neg, zero, pos, neg+z, not_z, pos+z, num,   var,
		    atm, str, top]]).

% @< ,[As...],
% B , X1
% ...
% means A < B -> newSubsA = X1. To know newSubsB check B > A
true_table([@<, [bot, neg, zero, pos, neg+z, not_z, pos+z, num, var, atm, str,
		    top],
		[bot,   bot, bot, bot,  bot, bot,   bot,   bot,   bot, bot, bot,
		    bot, bot],
		[neg,   bot, neg, bot,  bot, neg,   neg,   bot,   neg, var, bot,
		    bot, top],
		[zero,  bot, neg, bot,  bot, neg,   neg,   bot,   neg, var, bot,
		    bot, top],
		[pos,   bot, neg, zero, pos, neg+z, not_z, pos+z, num, var, bot,
		    bot, top],
		[neg+z, bot, neg, bot,  bot, neg,   neg,   bot,   neg, var, bot,
		    bot, top],
		[not_z, bot, neg, zero, pos, neg+z, not_z, pos+z, num, var, bot,
		    bot, top],
		[pos+z, bot, neg, zero, pos, neg+z, not_z, pos+z, num, var, bot,
		    bot, top],
		[num,   bot, neg, zero, pos, neg+z, not_z, pos+z, num, var, bot,
		    bot, top],
		[var,   bot, bot, bot,  bot, bot,   bot,   bot,   bot, var, bot,
		    bot, var],
		[atm,   bot, neg, zero, pos, neg+z, not_z, pos+z, num, var, atm,
		    bot, top],
		[str,   bot, neg, zero, pos, neg+z, not_z, pos+z, num, var, atm,
		    str, top],
		[top,   bot, neg, zero, pos, neg+z, not_z, pos+z, num, var, atm,
		    str, top]]).

% @> ,[As...],
% B , X1
% ...
% means A > B -> newSubsA = X1. To know newSubsB check B < A
true_table([@>, [bot, neg, zero, pos, neg+z, not_z, pos+z, num, var, atm, str,
		    top],
		[bot,   bot, bot, bot,  bot, bot,   bot,   bot,   bot, bot, bot,
		    bot, bot],
		[neg,   bot, neg, zero, pos, neg+z, not_z, pos+z, num, bot, atm,
		    str, top],
		[zero,  bot, bot, bot,  pos, bot,   pos,   pos,   pos, bot, atm,
		    str, top],
		[pos,   bot, bot, bot,  pos, bot,   pos,   pos,   pos, bot, atm,
		    str, top],
		[neg+z, bot, neg, zero, pos, neg+z, not_z, pos+z, num, bot, atm,
		    str, top],
		[not_z, bot, neg, zero, pos, neg+z, not_z, pos+z, num, bot, atm,
		    str, top],
		[pos+z, bot, bot, bot,  pos, bot,   pos,   pos,   pos, bot, atm,
		    str, top],
		[num,   bot, neg, zero, pos, neg+z, not_z, pos+z, num, bot, atm,
		    str, top],
		[var,   bot, neg, zero, pos, neg+z, not_z, pos+z, num, var, atm,
		    str, top],
		[atm,   bot, bot, bot,  bot, bot,   bot,   bot,   bot, bot, atm,
		    str, top],
		[str,   bot, bot, bot,  bot, bot,   bot,   bot,   bot, bot, bot,
		    str, str],
		[top,   bot, neg, zero, pos, neg+z, not_z, pos+z, num, var, atm,
		    str, top]]).

true_table([lub, [bot, neg, zero, pos, neg+z, not_z, pos+z, num, var, atm, str,
		    top],
		[bot,   bot,   neg,   zero,  pos,   neg+z, not_z, pos+z, num,
		    var, atm, str, top],
		[neg,   neg,   neg,   neg+z, not_z, neg+z, not_z, num,   num,
		    top, top, top, top],
		[zero,  zero,  neg+z, zero,  pos+z, neg+z, num,   pos+z, num,
		    top, top, top, top],
		[pos,   pos,   not_z, pos+z, pos,   num,   not_z, pos+z, num,
		    top, top, top, top],
		[neg+z, neg+z, neg+z, neg+z, num,   neg+z, num,   num,   num,
		    top, top, top, top],
		[not_z, not_z, not_z, num,   not_z, num,   not_z, num,   num,
		    top, top, top, top],
		[pos+z, pos+z, num,   pos+z, pos+z, num,   num,   pos+z, num,
		    top, top, top, top],
		[num,   num,   num,   num,   num,   num,   num,   num,   num,
		    top, top, top, top],
		[var,   var,   top,   top,   top,   top,   top,   top,   top,
		    var, top, top, top],
		[atm,   atm,   top,   top,   top,   top,   top,   top,   top,
		    top, atm, top, top],
		[str,   str,   top,   top,   top,   top,   top,   top,   top,
		    top, top, str, top],
		[top,   top,   top,   top,   top,   top,   top,   top,   top,
		    top, top, top, top]]).

true_table([glb|X]) :- true_table([== |X]).
