:- use_package(library(tabling)).
:- use_package(library(abs_solver)).
:- use_module(library(prolog_sys)).
:- use_module(library(write)).
:- use_module(library(format)).
:- use_module(library(terms_check)).
:- use_module(library(aggregates)).

:- include(stored_programs).

:- include(mainfile).


cputime(T) :-
  statistics(runtime,[T,_]).

writeln(T) :-
  write(T),
  nl.

go :- 
  cputime(Start),
      analyze(permute(_A1,_A2,_A3,_A4,_A5,_A6,_A7,_A8,_A9,_A10)),
  cputime(End),
  T is End - Start,
  write('% permute_10.pl: execution time = '), format('~2f',T), write(' milliseconds'),nl.

:- table step_goal_table/3.

analyze(Name/L) :- !,
	length(List,L),
	inizialize(List),
	Goal =.. [Name | List],
	(
	    step_att(Goal) ->
	    true
	;
	    make_bot(List)
	),
	abolish_all_tables.

analyze(Goal) :-
	Goal =.. [_ | List],
	inizialize(List),
	(
	    step_att(Goal) ->
	    true
	;
	    make_bot(List)
	),
	abolish_all_tables.

inizialize([]).
inizialize([X | Xs]) :-
	abs(X, top),
	inizialize(Xs).

dump_info(In, Out) :-
	get_abs_list(In, Out).

interp_att(Goal, InfoIn, InfoOut) :-
	Goal =..[_ | List],
	put_abs_list(List, InfoIn),
	(
	    step_att(Goal) ->
	    true
	;
	    make_bot(List)
	),
	get_abs_list(List, InfoOut),
	abolish_all_tables.

interp_body_att([]).
interp_body_att([SubGoal | RestBody]) :-
	step_att(SubGoal),
	interp_body_att(RestBody).

step_goal_att(Goal) :-
	Goal =.. [_ | List],
	get_abs_list(List, InfoIn),
	step_goal_table(Goal, InfoIn, InfoOut),
	put_abs_list(_, InfoOut).

step_goal_table(Goal, InfoIn, InfoOut) :-
	Goal =.. [_ | List],
	put_abs_list(_, InfoIn),
	stored_program(Goal, Body),
	interp_body_att(Body),
	get_abs_list(List, InfoOut).


step_att(Goal) :-
	Goal =.. [_ | List],
	findall(List, 
	(
	    step_goal_att(Goal)
	), GoalClauses),
	join_info_att(GoalClauses, List).

step_att(V is E) :- 	V abs_is E.

step_att(atom(V)) :- 	abs_atom(V).

step_att(V1 = V2) :- 	V1 .=. V2.

step_att(V1 < V2):- 	V1 .<. V2.
step_att(V1 > V2):- 	V1 .>. V2.
step_att(V1 =:= V2):- 	V1 .=:=. V2.

step_att(V1 @< V2):- 	V1 .@<. V2.
step_att(V1 @> V2):- 	V1 .@>. V2.
step_att(V1 == V2):- 	V1 .==. V2.

step_att(V1 .<-. AbsV1) :- 	V1 .<-. AbsV1.
