% :- module(dist_constraints_min,[main/0, main/1]).
% :- use_package(library(default)).
:- include(mainfile).

:- use_package(library(tabling)).
:- active_tclp.


cputime(T) :-
  statistics(runtime,[T,_]).

writeln(T) :-
  write(T),
  nl.

go :- 
  cputime(Start),
  dist('A', 'B', D),  
  cputime(End),
  T is End - Start,
  write('% dist_constraints_min.pl: execution time = '), format('~2f',T), write(' milliseconds'), display('   dist(A, B, D)   D = '), value(D,VD), display(VD), nl.

:- table dist/3.

:- op(700, xfx, [new_is, 'new_<']).

dist(A,B,D) :-
	cost(A,C,C1),
	dist(C,B,C2),
	D 'new_is' C1 + C2.
dist(A,B,D) :-
	cost(A,B,C1),
	D 'new_is' C1.

size(10).

cost('A', 1, 10).
cost('A', 1, 30).
cost(n,'B', 40).
cost(n,1, 20).
cost(1,n, 25).
% cost(1,2,23).
% cost(2,3,23).
% cost(3,4,23).
% cost(4,5,23).
% cost(5,6,23).
% cost(6,7,23).
% cost(7,8,23).
% cost(8,9,23).
% cost(9,10,23).
% cost(10,n,23).
cost(X,Y,C) :-
	size(L),
	between(1, L - 1, X),
	Y is X + 1,
	C = 23.
cost(L,n,23) :- size(L).


%% Constraint solver %%
X 'new_is' Exp :-
	Exp =.. [Op, O1, O2], !,
	value(O1, V1),
	value(O2, V2),
	VExp =.. [Op, V1, V2],
	VX is VExp,
	put_attr(X, 'user',VX).
X 'new_is' V :-
	ground(V),
	put_attr(X, 'user',V).
X 'new_<' Y :-
	value(X, Vx),
	value(Y, Vy),
	Vx < Vy.
value(X,X) :- ground(X), !.
value(X,V) :- get_attr(X,'user',V).

:- use_module(library(attr/attr_rt)).
:- use_module(engine(attributes)).
:- multifile attr_portray_hook/2.
:- multifile attr_unify_hook/2. %% needed
:- multifile attribute_goals/3.
attr_unify_hook(Att, X) :-
	(
	    get_attr(X, 'user', Att1) ->
	    Att = Att1
	;
	    true
	).
attribute_goals(X, [S|T], T) :-
	get_attr(X, 'user',Att),
	S = (X is Att).
%% Constraint solver %%


%% Tabling interface %%
call_domain_projection([],[]).
call_domain_projection([X],[V]) :-
	get_attr(X,'user',V).
answer_domain_projection([],[]).
answer_domain_projection([X],[V]) :-
	get_attr(X,'user',V).

call_store_projection(_, _, _).
answer_store_projection(_, _, _).

entail(_, _, _, _).
% answer_check_entail(_,_,_,_,_,_):- fail.
answer_check_entail(_, [X1], [X2], _, Result,_) :-
	(
	    X1 >= X2 ->     %% entail looking for the minimal
	    Result = 1
	;
	    X1 < X2 ->      %% discard previous answer to obtaine the minimal
	    Result = -1
	;
	    Result = 0      %% Not necessary...
	).
apply_answer([],_,_).
apply_answer([X], [V], _) :-
	put_attr(X,'user',V).
%% Tabling interface %%



