% :- module(reach_constraints_min,[main/0, main/1]).
% :- use_package(library(default)).
:- include(mainfile).

:- use_package(library(tabling)).
:- active_tclp.


cputime(T) :-
  statistics(runtime,[T,_]).

writeln(T) :-
  write(T),
  nl.

go :- 
  cputime(Start),
  reach('A', 'B', D),  
  cputime(End),
  T is End - Start,
  write('% reach_constraints_min.pl: execution time = '), format('~2f',T), write(' milliseconds'), display('   reach(A, B, D)   D = '), value(D,VD), display(VD), nl.

:- table reach/3.

:- op(700, xfx, [new_is, 'new_<']).

reach(A,B,D) :-
	edge(A,C),
	reach(C,B,D1),
	D 'new_is' D1 + 1.
reach(A,B,D) :-
	edge(A,B),
	D 'new_is' 1.

size(10).

edge('A',1).
edge(n,'B').
edge(n,1).
edge(1,n).
% edge(1,2).
% edge(2,3).
% edge(3,4).
% edge(4,5).
% edge(5,6).
% edge(6,7).
% edge(7,8).
% edge(8,9).
% edge(9,10).
% edge(10,n).
 edge(X,Y) :-
	size(L),
	between(1, L - 1, X),
	Y is X + 1.
edge(L,n) :- size(L).


%% Constraint solver %%
X 'new_is' Exp :-
	Exp =.. [Op, O1, O2], !,
	value(O1, V1),
	value(O2, V2),
	VExp =.. [Op, V1, V2],
	VX is VExp,
	put_attr(X, 'user',VX).
X 'new_is' V :-
	ground(V),
	put_attr(X, 'user',V).
X 'new_<' Y :-
	value(X, Vx),
	value(Y, Vy),
	Vx < Vy.
value(X,X) :- ground(X), !.
value(X,V) :- get_attr(X,'user',V).

:- use_module(library(attr/attr_rt)).
:- use_module(engine(attributes)).
:- multifile attr_portray_hook/2.
:- multifile attr_unify_hook/2. %% needed
:- multifile attribute_goals/3.
attr_unify_hook(Att, X) :-
	(
	    get_attr(X, 'user', Att1) ->
	    Att = Att1
	;
	    true
	).
attribute_goals(X, [S|T], T) :-
	get_attr(X, 'user',Att),
	S = (X is Att).
%% Constraint solver %%


%% Tabling interface %%
call_domain_projection([],[]).
call_domain_projection([X],[V]) :-
	get_attr(X,'user',V).
answer_domain_projection([],[]).
answer_domain_projection([X],[V]) :-
	get_attr(X,'user',V).

call_store_projection(_, _, _).
answer_store_projection(_, _, _).

entail(_, _, _, _).
% answer_check_entail(_,_,_,_,_,_):- fail.
answer_check_entail(_, [X1], [X2], _, Result,_) :-
	(
	    X1 >= X2 ->     %% entail looking for the minimal
	    Result = 1
	;
	    X1 < X2 ->      %% discard previous answer to obtaine the minimal
	    Result = -1
	;
	    Result = 0      %% Not necessary...
	).
apply_answer([],_,_).
apply_answer([X], [V], _) :-
	put_attr(X,'user',V).
%% Tabling interface %%

