:- package(tab_aggregates).

:- use_package(attr).
:- use_module(library(terms_check)).


make_attr(Pred, AttPred, Calls) :-
	Pred =.. [X|Arg],
	put_in_var(Arg, AttArg, Calls),
	AttPred =.. [X|AttArg].

undo_attr(AttPred, Pred) :-
	AttPred =.. [N|AttArg],
	get_from_var(AttArg, Arg, _),
	Pred =.. [N|Arg].

put_in_var([],     [],     []).
put_in_var([X|Xs], [X|Ns], [C|Cs]) :-
	var(C), !,
	put_in_var(Xs, Ns, Cs).
put_in_var([X|Xs], [N|Ns], [C|Cs]) :-
	var(N),
	put_attr_local(N, info(X, C)),
	put_in_var(Xs, Ns, Cs).

get_from_var([],     [],     []).
get_from_var([X|Xs], [N|Ns], [C|Cs]) :-
	get_attr_local(X, info(N, C)), !,
	get_from_var(Xs, Ns, Cs).
get_from_var([X|Xs], [X|Ns], [_|Cs]) :-
	get_from_var(Xs, Ns, Cs).

%% Tabling interface %%
call_domain_projection(Var, Dom) :-               get_from_var(Var, Dom, _).
call_entail(_, [], [], _Calls).
call_entail(_, [_|Da], [_|Db], [nt |Calls]) :- !, call_entail(_, Da, Db, Calls).
call_entail(_, [A|Da], [B|Db], [sub|Calls]) :- !, 
	subsumes_term(B,A), 
	call_entail(_, Da, Db, Calls).
call_entail(_, [A|Da], [B|Db], [_  |Calls]) :-    variant(A, B), call_entail(_, Da, Db, Calls).
call_store_projection(Var, _, Calls) :-           get_from_var(Var, _, Calls).

answer_domain_projection(Var, Dom) :-            get_from_var(Var, Dom, _).
answer_check_entail(_, Da, Db, Calls, new, Dn) :- member(_/3,Calls), lattice_entail(Da, Db, Calls, Dn), !.
answer_check_entail(_, Da, Db, Calls,  1, _) :-  check_entail(Da,Db,Calls), !.
answer_check_entail(_, Da, Db, Calls, -1, _) :-  check_entail(Db,Da,Calls).
answer_store_projection(Var, _, Calls) :-        get_from_var(Var, _, Calls).

apply_answer(Var, Dom, _) :-                     apply_list(Var, Dom).
%% Tabling interface %%

apply_list([],     []).
apply_list([X|Xs], [D|Ds]) :-
	attr_unify_hook(info(D, _), X),
	apply_list(Xs, Ds).


check_entail([],         _,            _).
check_entail([_|Vs], [_|V0s], [nt |Calls]) :- !,
	check_entail(Vs, V0s, Calls).
check_entail([Value|Vs], [Value0|V0s], [sub |Calls]) :- !,
	subsumes_term(Value0, Value),
	check_entail(Vs, V0s, Calls).
check_entail([Value|Vs], [Value0|V0s], [Name/2|Calls]) :-
	Call =.. [Name, Value0, Value],
	call(Call),
	check_entail(Vs, V0s, Calls).

lattice_entail([], _, _, []).
lattice_entail([_|Vs], [V|V0s], [nt|Calls], [V|News]) :-
	lattice_entail(Vs, V0s, Calls, News).
lattice_entail([Value|Vs], [Value0|V0s], [Name/3|Calls], [NewValue|News]) :-
	Call =.. [Name, Value0, Value, NewValue],
	call(Call),
	lattice_entail(Vs, V0s, Calls, News).

% :- multifile attr_unify_hook/2. %% needed
attr_unify_hook(Att, X) :-
	get_attr_local(X, Att1),
	Att = Att1.
